;----------------------------------------------------------------------------
lcd_command8:	;used for init (we need some 8-bit commands to switch to 4-bit mode!)
	in	temp, DDRD		;we need to set the high nibble of DDRD while leaving
					;the other bits untouched. Using temp for that.
	sbr	temp, 0b11110000	;set high nibble in temp
	out	DDRD, temp		;write value to DDRD again
	in	temp, PortD		;then get the port value
	cbr	temp, 0b11110000	;and clear the data bits
	cbr	buf, 0b00001111	;then clear the low nibble of the buf
					;so that no control line bits are overwritten
	or	temp, buf		;then set the data bits (from the buf) in the
					;Port value
	out	PortD, temp		;and write the port value.
	sbi	PortD, LCD_E		;now strobe E
	nop
	nop
	nop
	cbi	PortD, LCD_E
	in	temp, DDRD		;get DDRD to make the data lines input again
	cbr	temp, 0b11110000	;clear data line direction bits
	out	DDRD, temp		;and write to DDRD
	rcall	LCD_wait
	ret
lcd_putchar:
	push	buf		;save the argmuent (it's destroyed in between)
	in	temp, DDRD		;get data direction bits
	sbr	temp, 0b11110000	;set the data lines to output
	out	DDRD, temp		;write value to DDRD
	in	temp, PortD		;then get the data from PortD
	cbr	temp, 0b11111110	;clear ALL LCD lines (data and control!)
	cbr	buf, 0b00001111	;we have to write the high nibble of our buf first
					;so mask off the low nibble
	or	temp, buf		;now set the buf bits in the Port value
	out	PortD, temp		;and write the port value
	sbi	PortD, LCD_RS		;now take RS high for LCD char data register access
	sbi	PortD, LCD_E		;strobe Enable
	nop
	nop
	nop
	cbi	PortD, LCD_E
	pop	buf		;restore the buf, we need the low nibble now...
	cbr	temp, 0b11110000	;clear the data bits of our port value
	swap	buf		;we want to write the LOW nibble of the buf to
					;the LCD data lines, which are the HIGH port nibble!
	cbr	buf, 0b00001111	;clear unused bits in buf
	or	temp, buf		;and set the required buf bits in the port value
	out	PortD, temp		;write data to port
	sbi	PortD, LCD_RS		;again, set RS
	sbi	PortD, LCD_E		;strobe Enable
	nop
	nop
	nop
	cbi	PortD, LCD_E
	cbi	PortD, LCD_RS
	in	temp, DDRD
	cbr	temp, 0b11110000	;data lines are input again
	out	DDRD, temp
	rcall	LCD_wait
	ret
lcd_command:	;same as LCD_putchar, but with RS low!
	push	buf
	in	temp, DDRD
	sbr	temp, 0b11110000
	out	DDRD, temp
	in	temp, PortD
	cbr	temp, 0b11111110
	cbr	buf, 0b00001111
	or	temp, buf
	out	PortD, temp
	sbi	PortD, LCD_E
	nop
	nop
	nop
	cbi	PortD, LCD_E
	pop	buf
	cbr	temp, 0b11110000
	swap	buf
	cbr	buf, 0b00001111
	or	temp, buf
	out	PortD, temp
	sbi	PortD, LCD_E
	nop
	nop
	nop
	cbi	PortD, LCD_E
	in	temp, DDRD
	cbr	temp, 0b11110000
	out	DDRD, temp
	rcall	LCD_wait
	ret
LCD_getaddr:	;works just like LCD_getchar, but with RS low, return.7 is the busy flag
	in	temp, DDRD
	andi	temp, 0b00001111
	out	DDRD, temp
	cbi	PortD, LCD_RS
	sbi	PortD, LCD_RW
	sbi	PortD, LCD_E
	nop
	in	temp, PinD
	andi	temp, 0b11110000
	mov	return, temp
	cbi	PortD, LCD_E
	nop
	nop
	sbi	PortD, LCD_E
	nop
	in	temp, PinD
	andi	temp, 0b11110000
	swap	temp
	or	return, temp
	cbi	PortD, LCD_E
	cbi	PortD, LCD_RW
	ret
LCD_wait:				;read address and busy flag until busy flag cleared
	rcall	LCD_getaddr
;	ret	;#####
	wdr
	andi	return, 0x80
	brne	LCD_wait
	ret
LCD_delay:	;50 msek
	clr	r2
	LCD_delay_outer:
	clr	r3
	LCD_delay_inner:
	dec	r3
	brne	LCD_delay_inner
	dec	r2
	brne	LCD_delay_outer
	ret
LCD_init:
	ldi	temp, 0b00001110	;control lines are output, rest is input
	out	DDRD, temp
	rcall	LCD_delay		;first, we'll tell the LCD that we want to use it
	rcall	LCD_delay		;first, we'll tell the LCD that we want to use it
	rcall	LCD_delay		;first, we'll tell the LCD that we want to use it
	rcall	LCD_delay		;first, we'll tell the LCD that we want to use it
	ldi	buf, 0x20		;in 4-bit mode.
	rcall	LCD_command8		;LCD is still in 8-BIT MODE while writing this command!!!
	ldi	buf, 0x28		;NOW: 2 lines, 5*7 font, 4-BIT MODE!
	rcall	LCD_command		;
	ldi	buf, 0x0C		;now proceed as usual: Display on, cursor off, blinking off
	rcall	LCD_command
	ldi	buf, 0x01		;clear display, cursor -> home
	rcall	LCD_command
	ldi	buf, 0x06		;auto-inc cursor
	rcall	LCD_command
	ret
